"""
Script: Betongbjelke Master utgangspunkt

Input file for NLFEA of beam master in 

Author: Daniel Markali

"""
import os
import math
import geometry_study2 

# Open new project
closeProject()
newProject( "Concretbeam master beam RC1.5", 10 )
setModelAnalysisAspects( [ "STRUCT" ] )
setModelDimension( "2D" )
setDefaultMeshOrder( "QUADRATIC" )
setDefaultMesherType( "HEXQUAD" )
setUnit( "LENGTH", "MM" )
setUnit( "FORCE", "N" )

# Set parameters
reinforcement_diameter = 20 #Angir hvilken armeringsdiameter for hovedstrekkarmering 
multilineær = 1 # 1 betyr beregninger med multileneær kurve for betong i strekk, 0 gir eksponentiell. 


LS = 412.5 #Left span 
RS = LS #Right span 
MS = 500 #Midspan not all beams
overhang = 150
L = 2*overhang + LS + RS + MS #Total lengde bjelke
height = 300 #All beams
width = 150 #All beams
cover = 25 #All beams 
t = 30 #Halve opplegslengden Alle bjelker

element_size = 30 #All beams

distans_between_stirrups = 100 #All beams 
nro = 3 #All beams

#Concrete properties
youngs_modulus_con = 33000 #N/mm^2
poisson_ratio_con = 0.2 #All beams
density_con = 2.50e-9 #T/mm^3 #All beams 
compressive_strength_con = 23.80 #N/mm^2
tensile_strength_con = 2.42 #N/mm^2
fracture_energy_tension = 0.073*compressive_strength_con**0.18 #All beams with exsponential tention curve
fracture_energy_compression = fracture_energy_tension*250 #All beams

if multilineær == 1: #Verdiene må endres for hver betong
    eps0 = (0/1000)/element_size
    eps1 = (568/1000)/element_size
    eps2 = (664/1000)/element_size
    eps3 = (764/1000)/element_size
    eps4 = (864/1000)/element_size
    eps5 = (964/1000)/element_size
    eps6 = (1022/1000)/element_size
    eps7 = (1022/1000)/element_size
    
    
    sig0 = 0
    sig1 = 2.42
    sig2 = 2.37
    sig3 = 2.32
    sig4 = 2.10
    sig5 = 1.80
    sig6 = 1.52
    sig7 = 0

    tensile_strength_vector = [eps0,sig0,eps1,sig1,eps2,sig2,eps3,sig3,eps4,sig4,eps5,sig5,eps6,sig6,eps7,sig7]

#Material Steel Plate
youngs_modulus_plate = 210000 #N/mm^2 #All beams
poisson_ratio_plate = 0.3 #All beams
yield_strength_plate = 355 #N/mm^2 #All beams

#Reinforcment properties
max_strain = 0.1 #All beams
#Steel
yield_strength_steel = 335.0 #N/mm^2 #All beams
youngs_modulus_steel = 210000 #N/mm^2 #All beams
hardening_modulus_steel = 0.02*youngs_modulus_steel #All beams
stress_max_steel = yield_strength_steel + hardening_modulus_steel*(max_strain - yield_strength_steel/youngs_modulus_steel) #All beams
strain_pl_max_steel = max_strain*(1-hardening_modulus_steel/youngs_modulus_steel) + yield_strength_steel/youngs_modulus_steel*(hardening_modulus_steel/youngs_modulus_steel - 1) #All beams

yield_strength_steel_shear = 235.0 #N/mm^2 #All beams
youngs_modulus_steel_shear = 210000 #N/mm^2 #All beams
hardening_modulus_steel_shear = 0.02*youngs_modulus_steel_shear #All beams
stress_max_steel_shear = yield_strength_steel_shear + hardening_modulus_steel_shear*(max_strain - yield_strength_steel_shear/youngs_modulus_steel_shear) #All beams
strain_pl_max_steel_shear = max_strain*(1-hardening_modulus_steel_shear/youngs_modulus_steel_shear) + yield_strength_steel_shear/youngs_modulus_steel_shear*(hardening_modulus_steel_shear/youngs_modulus_steel_shear - 1) #All beams

#longditudinal 
diameter_Y10 = 10
as_Y10 = math.pi/4*diameter_Y10**2
diameter_Y12 = 12
as_Y12 = math.pi/4*diameter_Y12**2
diameter_Y16 = 16
as_Y16 = math.pi/4*diameter_Y16**2
diameter_Y20 = 20
as_Y20 = math.pi/4*diameter_Y20**2
diameter_Y25 = 25
as_Y25 = math.pi/4*diameter_Y25**2

#shear
diameter_Y8_shear = 8
as_Y8_shear = math.pi/4*diameter_Y8_shear**2

#Geometry
if reinforcement_diameter == 12:
    shape_list,shape_number,plate_list,plate_number = geometry_study2.geometry_concrete(L,height,overhang,LS,RS,createSheet,t)
    Y12,Y8_shear = geometry_study2.geometry_reinforcement(L,height,cover,overhang,distans_between_stirrups,nro,createLine)

if reinforcement_diameter == 16:
    shape_list,shape_number,plate_list,plate_number = geometry_study2.geometry_concrete(L,height,overhang,LS,RS,createSheet,t)
    Y16,Y8_shear = geometry_study2.geometry_reinforcement(L,height,cover,overhang,distans_between_stirrups,nro,createLine)

if reinforcement_diameter == 20:
    shape_list,shape_number,plate_list,plate_number = geometry_study2.geometry_concrete(L,height,overhang,LS,RS,createSheet,t)
    Y20,Y8_shear = geometry_study2.geometry_reinforcement(L,height,cover,overhang,distans_between_stirrups,nro,createLine)

if reinforcement_diameter == 25:
    shape_list,shape_number,plate_list,plate_number = geometry_study2.geometry_concrete(L,height,overhang,LS,RS,createSheet,t)
    Y25,Y8_shear = geometry_study2.geometry_reinforcement(L,height,cover,overhang,distans_between_stirrups,nro,createLine)

fitAll()


#Material Concrete
addMaterial( "Concrete", "CONCR", "TSCR", [] )
setParameter( MATERIAL, "Concrete", "LINEAR/ELASTI/YOUNG", youngs_modulus_con )
setParameter( MATERIAL, "Concrete", "LINEAR/ELASTI/POISON", poisson_ratio_con )
setParameter( MATERIAL, "Concrete", "LINEAR/MASS/DENSIT", density_con )

setParameter( MATERIAL, "Concrete", "MODTYP/TOTCRK", "ROTATE" )

setParameter( MATERIAL, "Concrete", "COMPRS/COMCRV", "PARABO" )
setParameter( MATERIAL, "Concrete", "COMPRS/COMSTR", compressive_strength_con )
setParameter( MATERIAL, "Concrete", "COMPRS/GC", fracture_energy_compression )
setParameter( MATERIAL, "Concrete", "COMPRS/CONFIN/CNFCRV", "VECCHI" )
setParameter( MATERIAL, "Concrete", "COMPRS/REDUCT/REDCRV", "VC1993" )
setParameter( MATERIAL, "Concrete", "COMPRS/REDUCT/REDMIN", 0.4 ) 

if multilineær == 1:
    setParameter( MATERIAL , "Concrete", "TENSIL/TENCRV", "MULTLN" )
    setParameter( MATERIAL , "Concrete", "TENSIL/EPSIGT", tensile_strength_vector )

if multilineær == 0:

    setParameter( MATERIAL, "Concrete", "TENSIL/TENSTR", tensile_strength_con )
    setParameter( MATERIAL, "Concrete", "TENSIL/TENCRV", "EXPONE" )
    setParameter( MATERIAL, "Concrete", "TENSIL/GF1", fracture_energy_tension )
    setParameter( MATERIAL, "Concrete", "TENSIL/CBSPEC", "GOVIND" )

setParameter( MATERIAL, "Concrete", "TENSIL/POISRE/POIRED", "DAMAGE" )

con_mat_name = "Concrete"
addGeometry( con_mat_name, "SHEET", "MEMBRA", [] )
setParameter( GEOMET, con_mat_name, "THICK", width )

assignMaterial( con_mat_name, "SHAPE", shape_list )
assignGeometry( con_mat_name, "SHAPE", shape_list )


#Reinforcment materials
Y10_mat_name = 'Y10'
Y16_mat_name = 'Y16'
Y8_shear_mat_name = 'Y8_shear'
    

if reinforcement_diameter == 12:
    Y16_mat_name = 'Y12'
    as_Y = as_Y12 
    Y = Y12

    convertToReinforcement( Y12 )
    convertToReinforcement( Y8_shear )

if reinforcement_diameter == 16:
    Y16_mat_name = 'Y16'
    as_Y = as_Y16
    Y = Y16

    convertToReinforcement( Y16 )
    convertToReinforcement( Y8_shear )
    
if reinforcement_diameter == 20:
    Y16_mat_name = 'Y20'
    as_Y = as_Y20
    Y = Y20

    convertToReinforcement( Y20 )
    convertToReinforcement( Y8_shear )

if reinforcement_diameter == 25:
    Y16_mat_name = 'Y25'
    as_Y = as_Y20 
    Y = Y25

    convertToReinforcement( Y25 )
    convertToReinforcement( Y8_shear )
     

#Adding reinforcement materials
#Steel bars
addMaterial( Y16_mat_name, "REINFO", "VMISES", [] )
setParameter( MATERIAL, Y16_mat_name, "LINEAR/ELASTI/YOUNG", youngs_modulus_steel )
setParameter( MATERIAL, Y16_mat_name, "PLASTI/YLDTYP", "KAPSIG" )
setParameter( MATERIAL, Y16_mat_name, "PLASTI/HARDI2/KAPSIG", [] )
setParameter( MATERIAL, Y16_mat_name, "PLASTI/HARDI2/KAPSIG", [ 0, yield_strength_steel, strain_pl_max_steel, stress_max_steel ] )

addGeometry( Y16_mat_name, "RELINE", "REBAR", [] )
setParameter( GEOMET, Y16_mat_name, "REIEMB/RDITYP", "RCROSS" )
setParameter( GEOMET, Y16_mat_name, "REIEMB/CROSSE", as_Y*2 )

assignMaterial( Y16_mat_name, "SHAPE", Y )
assignGeometry( Y16_mat_name, "SHAPE", Y )


addMaterial( Y8_shear_mat_name, "REINFO", "VMISES", [] )
setParameter( MATERIAL, Y8_shear_mat_name, "LINEAR/ELASTI/YOUNG", youngs_modulus_steel_shear )
setParameter( MATERIAL, Y8_shear_mat_name, "PLASTI/YLDTYP", "KAPSIG" )
setParameter( MATERIAL, Y8_shear_mat_name, "PLASTI/HARDI2/KAPSIG", [] )
setParameter( MATERIAL, Y8_shear_mat_name, "PLASTI/HARDI2/KAPSIG", [ 0, yield_strength_steel_shear, strain_pl_max_steel_shear, stress_max_steel_shear ] )

addGeometry( Y8_shear_mat_name, "RELINE", "REBAR", [] )
setParameter( GEOMET, Y8_shear_mat_name, "REIEMB/RDITYP", "RCROSS" )
setParameter( GEOMET, Y8_shear_mat_name, "REIEMB/CROSSE", as_Y8_shear*2 )

assignMaterial( Y8_shear_mat_name, "SHAPE", Y8_shear )
assignGeometry( Y8_shear_mat_name, "SHAPE", Y8_shear )

#Material steel plate study 1
plate_mat_name = 'Plate Steel'
addMaterial( plate_mat_name, "MCSTEL", "ISOTRO", [] )
setParameter( MATERIAL, plate_mat_name, "LINEAR/ELASTI/YOUNG", 210000 )
setParameter( MATERIAL, plate_mat_name, "LINEAR/ELASTI/POISON", 0.3 )

addGeometry( plate_mat_name , "SHEET", "MEMBRA", [] )
setParameter( GEOMET, plate_mat_name, "THICK", width )

assignMaterial( plate_mat_name, "SHAPE", plate_list )
assignGeometry( plate_mat_name, "SHAPE", plate_list )


#Support conditions
#Left Support
createLineSupport( "Left Support", "Geometry support set 1" )
setParameter( GEOMETRYSUPPORT, "Left Support", "AXES", [ 1, 2 ] )
setParameter( GEOMETRYSUPPORT, "Left Support", "TRANSL", [ 1, 1, 0 ] )
setParameter( GEOMETRYSUPPORT, "Left Support", "ROTATI", [ 0, 0, 0 ] )
attach( GEOMETRYSUPPORT, "Left Support", plate_list[0], [[ overhang, -10, 0 ]] )

#Right Support
createLineSupport( "Right Support", "Geometry support set 1" )
setParameter( GEOMETRYSUPPORT, "Right Support", "AXES", [ 1, 2 ] )
setParameter( GEOMETRYSUPPORT, "Right Support", "TRANSL", [ 0, 1, 0 ] )
setParameter( GEOMETRYSUPPORT, "Right Support", "ROTATI", [ 0, 0, 0 ] )
attach( GEOMETRYSUPPORT, "Right Support", plate_list[1], [[L-overhang, -10, 0 ]] )

#Loading
#Loading left
#Prescribed loading - support for load 
createLineSupport( "Top Support1", "Geometry support set 1" )
setParameter( GEOMETRYSUPPORT, "Top Support1", "AXES", [ 1, 2 ] )
setParameter( GEOMETRYSUPPORT, "Top Support1", "TRANSL", [ 0, 1, 0 ] )
setParameter( GEOMETRYSUPPORT, "Top Support1", "ROTATI", [ 0, 0, 0 ] )
attach( GEOMETRYSUPPORT, "Top Support1", plate_list[2], [[ LS+overhang, height+20, 0 ]] )

#Prescribed loading - deformation
addSet( GEOMETRYLOADSET, "Geometry load case 1" )
createLineLoad( "Prescribed Deformation", "Geometry load case 1" )
setParameter( GEOMETRYLOAD, "Prescribed Deformation", "LODTYP", "DEFORM" )
setParameter( GEOMETRYLOAD, "Prescribed Deformation", "DEFORM/SUPP", "Top Support1" )
setParameter( GEOMETRYLOAD, "Prescribed Deformation", "DEFORM/TR/VALUE", -1 )
setParameter( GEOMETRYLOAD, "Prescribed Deformation", "DEFORM/TR/DIRECT", 2 )
attach( GEOMETRYLOAD, "Prescribed Deformation", plate_list[2], [[ LS+overhang, height+20, 0 ]] )

#Loading right
#Prescribed loading - support for load 
createLineSupport( "Top Support2", "Geometry support set 1" )
setParameter( GEOMETRYSUPPORT, "Top Support2", "AXES", [ 1, 2 ] )
setParameter( GEOMETRYSUPPORT, "Top Support2", "TRANSL", [ 0, 1, 0 ] )
setParameter( GEOMETRYSUPPORT, "Top Support2", "ROTATI", [ 0, 0, 0 ] )
attach( GEOMETRYSUPPORT, "Top Support2", plate_list[3], [[ LS+overhang+MS, height+20, 0 ]] )

#Prescribed loading - deformation
createLineLoad( "Prescribed Deformation2", "Geometry load case 1" )
setParameter( GEOMETRYLOAD, "Prescribed Deformation2", "LODTYP", "DEFORM" )
setParameter( GEOMETRYLOAD, "Prescribed Deformation2", "DEFORM/SUPP", "Top Support2" )
setParameter( GEOMETRYLOAD, "Prescribed Deformation2", "DEFORM/TR/VALUE", -1 )
setParameter( GEOMETRYLOAD, "Prescribed Deformation2", "DEFORM/TR/DIRECT", 2 )
attach( GEOMETRYLOAD, "Prescribed Deformation2", plate_list[3], [[ LS+overhang+MS, height+20, 0 ]] )

# Meshing
setElementSize( shape_list, element_size )
setElementClassType( shape_list, "MEMBRA" )

setElementSize( plate_list,element_size )
setElementClassType( plate_list, "MEMBRA")

generateMesh(shapes())

#Setup Analysis
analysis_name = "Nonlinear analysis"
command  = "Structural nonlinear"
addAnalysis( analysis_name )
addAnalysisCommand( analysis_name, "NONLIN", command )

setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/METHOD/METNAM", "NEWTON" )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/METHOD/NEWTON/TYPNAM", "REGULA" )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/MAXITE", 50 )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/CONVER/FORCE", True )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/CONVER/ENERGY", True )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/CONVER/DISPLA", False )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/CONVER/FORCE/TOLCON", 0.01 )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/CONVER/FORCE/NOCONV", "CONTIN" )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/CONVER/ENERGY/TOLCON", 0.001 )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/CONVER/ENERGY/NOCONV", "CONTIN" )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/ITERAT/CONVER/SIMULT", False )

setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/LOAD/STEPS/EXPLIC/SIZES", "0.1(300)" )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/LOGGIN/REPORT/AMOUNT", "FULL" )
setAnalysisCommandDetail( analysis_name, command, "EXECUT(1)/LOGGIN/REPORT/TERMIN", "STEP" )
setAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/SELTYP", "USER" )
addAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER" )
addAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/DISPLA" )
addAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/FORCE" )
addAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRAIN" )
setAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRAIN(1)/TOTAL/GREEN/GLOBAL/LOCATI", "INTPNT" )
addAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRAIN(2)/TOTAL/GREEN/PRINCI" )
setAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRAIN(2)/TOTAL/GREEN/PRINCI/LOCATI", "INTPNT" )
addAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRAIN(3)/CRACK/GREEN" )
addAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRESS" )
setAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRESS(1)/TOTAL/CAUCHY/GLOBAL/LOCATI", "INTPNT" )
addAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRESS(2)/TOTAL/CAUCHY/PRINCI" )
setAnalysisCommandDetail( analysis_name, command, "OUTPUT(1)/USER/STRESS(2)/TOTAL/CAUCHY/PRINCI/LOCATI", "INTPNT" )


# EXPORT .DAT- AND .DCF-FILE
exportModel( "Concretbeam master beam RC1.5.dat", 5 )
saveAnalysisCommands( analysis_name, "Concretbeam master beam RC1.5.dcf")
saveProjectAs( "C:/Mastergrad DM/Analyse/Data analyse/RC1.5/Concretbeam master beam RC1.5.dpf" )
saveProjectAs( "C:/Mastergrad DM/Analyse/Data analyse/RC1.5/Concretbeam master beam RC1.5.dat" )
saveProjectAs( "C:/Mastergrad DM/Analyse/Data analyse/RC1.5/Concretbeam master beam RC1.5.dcf" )

# RUN ANALYSIS
runSolver( analysis_name )
showView( "RESULT" )

#Setter innstillinger for fargeplott
setViewSettingValue( "view setting", "RESULT/CONTOU/LEGEND", "GRADIE" )
setViewSettingValue( "view setting", "RESULT/CONTOU/AUTRNG", "AUTVIS" )
setViewSettingValue( "view setting", "RESULT/CONTOU/AUTRNG", "LIMITS" )
setViewSettingValue( "view setting", "RESULT/CONTOU/LIMITS/MINVAL", -40 )
setViewSettingValue( "view setting", "RESULT/CONTOU/LIMITS/MAXVAL", 5 ) 




